local LMP = LibStub("LibMediaProvider-1.0")

local ROW_COLOR_NORMAL = "#b0b0b0"
local ROW_COLOR_DEFAULT_FONT = LMP:Fetch('font', "Univers 67") .. "|" .. 16 .. "|" .. "outline"

local ListBox = ZO_SortFilterList:Subclass()
function ListBox:New(_name, _parent, _headerList, _virtualControlName, _font, _rowDistance)
	self.itemList = {}
	self.ListBoxControl = CreateControlFromVirtual(_name, _parent, "ListBox") 
	
	local headerControl = GetControl(self.ListBoxControl, "Headers")
	local lastColumn = nil
	local count = 1
	
	for columnKey, columnData in pairs(_headerList) do
		local listBoxColumn = CreateControlFromVirtual(_name .. count, headerControl, "ListBoxHeaderColumn")
		
		ZO_SortHeader_Initialize(listBoxColumn, columnData.Text, columnKey, ZO_SORT_ORDER_DOWN, TEXT_ALIGN_LEFT, _font)
		
		local columnWidth = columnData.Width
		
		listBoxColumn:SetWidth(columnWidth)
		
		if lastColumn then
			listBoxColumn:SetAnchor(TOPLEFT, lastColumn, TOPRIGHT, 6, 0)
		else
			listBoxColumn:SetAnchor(TOPLEFT)
		end
		
		lastColumn = listBoxColumn
		count = count + 1
		
		listBoxColumn:SetMouseEnabled(columnData.AllowSort)
	end
	
	local manager = ZO_SortFilterList.New(self, self.ListBoxControl)

	local rowDistance = 30
	if _rowDistance then
		rowDistance = _rowDistance
	end	
	
 	ZO_ScrollList_AddDataType(manager.list, 1, _virtualControlName, rowDistance, 
	function(control, data) 
		self:SetupUnitRow(control, data) 
	end)	

	return manager
end

function ListBox:SortScrollList()
    if(self.currentSortKey ~= nil and self.currentSortOrder ~= nil) then
        local scrollData = ZO_ScrollList_GetDataList(self.list)
		
		table.sort(scrollData, 
		function(listEntry1, listEntry2) 
			if listEntry1 and listEntry2 then
				local data1 = listEntry1.data.items[self.currentSortKey]
				local data2 = listEntry2.data.items[self.currentSortKey]
				
				if data1.type and data2.type then				
					local value1 = 0
					local value2 = 0
				
					if data1.type == "string" then
						value1 = zo_strlower(data1.sortValue)
					elseif data1.type == "number" then
						value1 = tonumber(data1.sortValue)			
					end
													
					if data2.type == "string" then
						value2 = zo_strlower(data2.sortValue)
					elseif data2.type == "number" then
						value2 = tonumber(data2.sortValue)				
					end			

					if self.currentSortOrder then
						return value1 < value2
					end
				
					return value1 > value2
				else
					return false
				end
			end
		end)
    end
end

function ListBox:FilterScrollList()
    local scrollData = ZO_ScrollList_GetDataList(self.list)
    ZO_ClearNumericallyIndexedTable(scrollData)

    for i = 1, #self.masterList do
        local data = self.masterList[i]
    	table.insert(scrollData, ZO_ScrollList_CreateDataEntry(1, data))
    end    	
end

function ListBox:BuildMasterList()
	self.masterList = {}
	for rowKey, rowData in pairs(self.itemList) do
		self.masterList[rowKey] = {["items"] = rowData} 
		self.masterList[rowKey].rowKey = rowKey
	end		
end

function ListBox:SetupUnitRow(control, data)
	local color = 0
	local lastColumn = nil
	local rowKey = data.rowKey
	
	for key, data in pairs(data.items) do
		local control = control:GetChild(key)		
		if control then		
			local controlType = control:GetType()
			if controlType == CT_LABEL then
				if data.type == "count" then					
					control:SetText(rowKey .. ".")
				else
					control:SetText(data.value)
				end
				
				if data.font then
					control:SetFont(data.font)
				else
					control:SetFont(ROW_COLOR_DEFAULT_FONT)
				end
						
				if data.fontColor then
					control:SetColor(data.fontColor:UnpackRGBA())
				else
					control:SetColor(AUI.Color.ConvertHexToRGBA(ROW_COLOR_NORMAL, 1):UnpackRGBA())
				end					
			elseif controlType == CT_TEXTURE then
				control:SetTexture(data.value)				
			end			
			
			if data.width and data.height then
				control:SetDimensions(data.width, data.height)
			elseif data.width then
				control:SetWidth(data.width)
			else
				control:SetHeight(data.height)
			end

			if lastColumn then
				control:SetAnchor(TOPLEFT, lastColumn, TOPRIGHT, 6, 0)
			else
				control:SetAnchor(TOPLEFT, nil, nil, 0,0)
			end				
				
			lastColumn = control
		end	
	end
end

function ListBox:Refresh()
	self:RefreshData()
end

function ListBox:SetItemList(_list)
	self.itemList = {}
	
	local rowList = _list

	for rowKey, _ in pairs(rowList) do
		if not self.itemList[rowKey] then
			self.itemList[rowKey] = {}
		end
		for itemKey, itemName in pairs(rowList[rowKey]) do		
			self.itemList[rowKey][itemKey] = itemName			
		end
	end	

	self:Refresh()
end

function AUI.CreateListBox(_name, _parent, _headerList, _virtualControl, _font, _rowDistance)	
	local listBoxClass = ListBox:New(_name, _parent, _headerList, _virtualControl, _font, _rowDistance)
	listBoxClass.ListBoxControl.SetItemList = function(_list) listBoxClass:SetItemList(_list) end
	listBoxClass.ListBoxControl.Refresh = function() listBoxClass:Refresh() end
	return listBoxClass.ListBoxControl
end